%% MEDSEM - 3D Visualization Frontend for Vol3D
%
% Selects a directory of deconvolved MEDSEM images for 3D visualization
% as named  "slice01.tif" ... "sliceXX.tif" 
% Naming convention (among other options) can be changed below.
%
% M. de Goede, E. Johlin, AMOLF
% F. Boughorbel, FEI
% NOTE: Requires the Vol3D MATLAB code to run, available here:
%   http://nl.mathworks.com/matlabcentral/fileexchange/22940-vol3d-v2
% Last modified October, 2016
%% Inputs
clear all;
xscalefact=2.0; %in nm/pixel
yscalefact=2.0; %in nm/pixel
zscalefact=3.0; %in nm/slice
loadnamepre='slice';  %Determines the file name of the loaded file e.g. "slice01.tif"
%% Get data of image stack
loadpath = uigetdir; %select data folder
if loadpath == 0 % If the input dialogue is canceled, exit.
    disp('Canceled, exiting...');
    return;
end
Y1 = [];
disp('Loading images...');
loadcheck=2;
ii=1; %Determines first image to load
loadname=strcat('\',loadnamepre, sprintf('%02d',ii),'.tif'); %load file name, add zeroes to keep 2 digits
while loadcheck==2 % load all slice images
    Y1(:,:,ii) =255-imread([loadpath,loadname]);
    ii=ii+1;
    loadname=strcat('\',loadnamepre, sprintf('%02d',ii),'.tif');
    loadcheck=exist(strcat(loadpath,'/',loadname), 'file');
end;

Y1=(Y1)/255; % Normalize intensity to 1

Y1=flip(Y1,3);
Y1=flip(Y1,2); % Correct orientation

%% Optional structure processing
% Y1(1:ceil(end/2),:,:)=0;  % Optional slicing of 3D rendering
% Y1=smooth3(Y1,'gaussian',[3,3,3]); % Optional gaussian smoothing
% Y1=max(Y1-0.2,0); %Optional background level

%% Plot data
figure()
h = vol3d('cdata',Y1,'texture','2D','XData',xscalefact.*[0:size(Y1,2)-1],'YData',yscalefact.*[0:size(Y1,1)-1],'ZData',zscalefact.*[0:size(Y1,3)-1]);
view(3); 
vol3d(h);   
alphamap('rampup'), alphamap('decrease'), alphamap('decrease'); % Low signal regions are transparent

colormap(hot); % Bone works well as well
colormap(flipud(colormap)); % Correct colormap direction
daspect([1,1,1]); % Maintain aspect ratio

%% Set plot lables
set(gca,'FontSize',12);
xlhand = get(gca,'xlabel');
set(xlhand,'string','Position X [nm]','fontsize',12);
ylhand = get(gca,'ylabel');
set(ylhand,'string','Position Y [nm]','fontsize',12);
zlhand = get(gca,'zlabel');
set(zlhand,'string','Position Z [nm]','fontsize',12);
t=title('MEDSEM Reconstruction');
set(t, 'FontSize', 12);
axis tight;