%% MEDSEM - Estimation of Divergence of normalized gradient
% Support file for deconvolution - total variation reglarization
% X.Zhuge, FEI company
%%
function tv = TVE3D(Image,L,ht,wd)

%% numerical implementation
Image = reshape(Image, [L,ht,wd]); 

dxplus = zeros(size(Image));
for ii = 1:wd-1
    dxplus(:,:,ii) = Image(:,:,ii+1) - Image(:,:,ii);
end;

dxminus = zeros(size(Image));
for ii = 2:wd
    dxminus(:,:,ii) = Image(:,:,ii) - Image(:,:,ii-1);
end;

dyplus = zeros(size(Image));
for ii = 1:ht-1
    dyplus(:,ii,:) = Image(:,ii+1,:) - Image(:,ii,:);
end;

dyminus = zeros(size(Image));
for ii = 2:ht
    dyminus(:,ii,:) = Image(:,ii,:) - Image(:,ii-1,:);
end;

dzplus = zeros(size(Image));
for ii = 1:L-1
    dzplus(ii,:,:) = Image(ii+1,:,:) - Image(ii,:,:);
end;

dzminus = zeros(size(Image));
for ii = 2:L
    dzminus(ii,:,:) = Image(ii,:,:) - Image(ii-1,:,:);
end;

mx = min(abs(dxplus),abs(dxminus)).*(sign(dxplus)+sign(dxminus))/2;
my = min(abs(dyplus),abs(dyminus)).*(sign(dyplus)+sign(dyminus))/2;
mz = min(abs(dzplus),abs(dzminus)).*(sign(dzplus)+sign(dzminus))/2;

eps = 1e-9;
Sx = dxplus./sqrt(dxplus.^2 + my.^2 + mz.^2 +eps);
Sy = dyplus./sqrt(dyplus.^2 + mx.^2 + mz.^2 +eps);
Sz = dzplus./sqrt(dzplus.^2 + mx.^2 + my.^2 +eps);

Sxminus = zeros(size(Sx));
for ii = 2:wd
    Sxminus(:,:,ii) = Sx(:,:,ii) - Sx(:,:,ii-1);
end;

Syminus = zeros(size(Sy));
for ii = 2:ht
    Syminus(:,ii,:) = Sy(:,ii,:) - Sy(:,ii-1,:);
end;

Szminus = zeros(size(Sz));
for ii = 2:L
    Szminus(ii,:,:) = Sz(ii,:,:) - Sz(ii-1,:,:);
end;

tv = reshape((Sxminus + Syminus + Szminus),[L,ht*wd]);
